% *************************************************************************
%
% Copyright: (c)  Florida International University. All rights reserved.
%
% MODULE:         G.m
%
% COMPILER:       Matlab R2010a.
%
% LANGUAGE:       Matlab Interpreter.
%
% AUTHOR:         Arvind Merwaday, Florida International Unversity
%
% ABSTRACT:       This function evaluates the G function defined by eq.(19)
%                 of the paper.
%
% TIME-STAMP:     21 Mar. 2014
%
% =========================================================================
% Input Parameters:
% =========================================================================
% None
% 
% =========================================================================
% Output Parameters:
% =========================================================================
% None

function res = G(g,Ri,Params)

%% Unpack the input parameters
tau         = Params.tau;
rho         = Params.rho;
rhoPrime    = Params.rhoPrime;
lambda      = Params.lambda;
lambdaPrime = Params.lambdaPrime;
dmin        = Params.Dmin_macro;
dmin_p      = Params.Dmin_pico;

%% Set integration limits and tolerances
TOL = 1e-2; % Tolerance for integration error
maxlim = 10000; % maximum limit for SIRs during integration
dmax = sqrt(log(1e-20)/(-lambda*pi)); % r > dmax is highly unlikely
dmax_p = sqrt(log(1e-20)/(-lambdaPrime*pi)); % rPrime > dmax1 is highly unlikely

% gm --> gamma
% gmp --> gamma\prime
% See Fig. (5) for the integration limits of different regions
if(strcmp(Ri,'R1'))
    res = quad2d(@(gm,gmp) g(gm,gmp).*JPDFuncond(gm,gmp), rho, maxlim, 0, @(gm) min(1./gm,gm/tau),'AbsTol', TOL);
elseif(strcmp(Ri,'R2'))
    res = quad2d(@(gm,gmp) g(gm,gmp).*JPDFuncond(gm,gmp), 0, rho, 0, @(gm) min(1./gm,gm/tau),'AbsTol', TOL);
elseif(strcmp(Ri,'R3'))
    res = quad2d(@(gmp,gm) g(gm,gmp).*JPDFuncond(gm,gmp), 0,rhoPrime,0,@(gmp) min(tau*gmp,1./gmp),'AbsTol', TOL);
else
    res = quad2d(@(gmp,gm) g(gm,gmp).*JPDFuncond(gm,gmp), rhoPrime,maxlim,0,@(gmp) min(tau*gmp,1./gmp),'AbsTol', TOL);
end

%% Evaluate the unconditional JPDF as per eq.(17)
    function f = JPDFuncond(gm,gmp)
        f = zeros(size(gm));
        for m = 1:size(gm,1)
            for n = 1:size(gm,2)
                f(m,n) = dblquad(@(r,r1) 2*pi*lambda*r.*exp(-lambda*pi*r.^2) .* 2*pi*lambdaPrime*r1.*exp(-lambdaPrime*pi*r1.^2) .* JPDFcond_PPP(gm(m,n),gmp(m,n),r,r1,Params), dmin, dmax, dmin_p, dmax_p, TOL);
            end
        end
    end

end