% *************************************************************************
%
% Copyright: (c)  Florida International University. All rights reserved.
%
% MODULE:         JPDFcond_PPP.m
%
% COMPILER:       Matlab R2010a.
%
% LANGUAGE:       Matlab Interpreter.
%
% AUTHOR:         Arvind Merwaday, Florida International Unversity
%
% ABSTRACT:       This function calculates the conditional JPDF of \Gamma
%                 and \Gamma\prime for the given input parameters using 
%                 Appendix 2 of the paper
%
% TIME-STAMP:     20 Mar. 2014
%
% =========================================================================
% Input Parameters:
% =========================================================================
% None
% 
% =========================================================================
% Output Parameters:
% =========================================================================
% None

function f = JPDFcond_PPP(gm,gmp,r,r1,Params)
%% Unpack the input parameters
alpha       = Params.alpha;
beta        = Params.beta;
l           = Params.lambda;
lp          = Params.lambdaPrime;
P_mac_dBm   = Params.P_mac_dBm;
P_pic_dBm   = Params.P_pic_dBm;
K_m_dB      = Params.K_m_dB;
K_p_dB      = Params.K_p_dB;

%% Calculate net transmit powers from MBSs and PBSs
P_mac = 10^(P_mac_dBm/10)/1000 * 10^(K_m_dB/10);
P_pic = 10^(P_pic_dBm/10)/1000 * 10^(K_p_dB/10);

%% evaluate JPDF
if(gm*gmp < 1 && gm*gmp>0)
    % global data
    acap = 1./(1 + (r1./r).^4*P_mac/P_pic);%
    mucap = 1/(1 + (lp/l)*sqrt(P_pic/P_mac));
    B = pi*r.^2./sqrt(P_mac*acap)*(l*sqrt(P_mac)+lp*sqrt(P_pic));%

    M1 = (1-gm*gmp)./((1+gm*acap./(1-acap)).*(1 + gmp*(1-acap)./acap));%
    c = sqrt((gm*(1+gmp)*acap + gmp*(1+gm)*(1-acap))/(1-gm*gmp));%
    M2 = exp(-B.*c.*(beta*mucap*(pi/2-atan(sqrt(acap)./c)) + (1-beta)*mucap*sqrt(alpha)*(pi/2-atan(sqrt(acap/alpha)./c)) + (1-mucap)*(pi/2-atan(sqrt(1-acap)./c))));%

    dM1dx = -acap.*(1-acap)./(acap*gm+1-acap).^2;%
    dM1dy = -acap.*(1-acap)./((1-acap)*gmp+acap).^2;%
    dcdx = 1/(2*gm*(1-gm*gmp)) *(c-gmp*(1-acap)./c);%
    dcdy = 1/(2*gmp*(1-gm*gmp)) *(c-gm*acap./c);%
    d2c = 1/(4*(1-gm*gmp)^2) *(3*c + 1./c - acap.*(1-acap)./c.^3);%
    h = log(M2)./c - B.*c.*(beta*mucap*acap.^(1/2)./(c.^2+acap) + (1-beta)*mucap*alpha*acap.^(1/2)./(c.^2*alpha+acap) + (1-mucap)*(1-acap).^(1/2)./(c.^2+1-acap));%
    dhdc = -2*B.*(beta*mucap*acap.^(3/2)./(c.^2+acap).^2 + (1-beta)*mucap*alpha*acap.^(3/2)./(c.^2*alpha+acap).^2 + (1-mucap)*(1-acap).^(3/2)./(c.^2+1-acap).^2);%

    f = M2.*h.*(dM1dx.*dcdy + dM1dy.*dcdx + d2c.*M1) + M1.*M2.*dcdx.*dcdy.*(h.^2+dhdc);
else
    f = zeros(size(r));
end

f(isnan(f)) = 0;
end