% *************************************************************************
%
% Copyright: (c)  Florida International University. All rights reserved.
%
% MODULE:         SpectralEfficiency_PPP.m
%
% COMPILER:       Matlab R2010a.
%
% LANGUAGE:       Matlab Interpreter.
%
% AUTHOR:         Arvind Merwaday, Florida International Unversity
%
% ABSTRACT:       This function calculates the per-user SEs of USF-MUEs
%                 and CSF-MUEs as per equations (32) and (33) of the paper.
%
% TIME-STAMP:     21 Mar. 2014
%
% =========================================================================
% Input Parameters:
% =========================================================================
% None
% 
% =========================================================================
% Output Parameters:
% =========================================================================
% None

function PerUserSE = SpectralEfficiency_PPP(Params)

%% Unpack the input parameters
alpha       = Params.alpha;
beta        = Params.beta;
lambda      = Params.lambda;
lambdaPrime = Params.lambdaPrime;

%% Calculate MUE and PUE probabilities as per eqs.(20)-(23) in the paper
Pmue_usf = G(@(gm,gmp) 1,'R2',Params); % eq.(21)
Pmue_csf = G(@(gm,gmp) 1,'R1',Params); % eq.(20)
Ppue_usf = G(@(gm,gmp) 1,'R4',Params); % eq.(22)
Ppue_csf = G(@(gm,gmp) 1,'R3',Params); % eq.(23)

%% Aggregate throughputs: G functions in the eqs.(28)-(31)
G_macusf = G(@(gm,gmp) log2(1+gm),'R2',Params);
G_maccsf = G(@(gm,gmp) log2(1+alpha*gm),'R1',Params);
G_picusf = G(@(gm,gmp) log2(1+gmp),'R4',Params);
G_piccsf = G(@(gm,gmp) log2(1 + gmp.*(1+gm)./(1+gm.*(alpha.*(gmp+1)-gmp))),'R3',Params);

% %% Mean number of MUEs and PUEs; eqs.(26)-(27)
% Nmue_usf = Pmue_usf/lambda;
% Nmue_csf = Pmue_csf/lambda;
% Npue_usf = Ppue_usf/lambdaPrime;
% Npue_csf = Ppue_csf/lambdaPrime;

%% Equate any NaNs to zero
Pmue_usf(isnan(Pmue_usf)) = 0;
Pmue_csf(isnan(Pmue_csf)) = 0;
Ppue_usf(isnan(Ppue_usf)) = 0;
Ppue_csf(isnan(Ppue_csf)) = 0;
G_macusf(isnan(G_macusf)) = 0;
G_maccsf(isnan(G_maccsf)) = 0;
G_picusf(isnan(G_picusf)) = 0;
G_piccsf(isnan(G_piccsf)) = 0;

%% Aggregate capacities; eqs.(28)-(31)
C_macusf = beta*G_macusf/Pmue_usf;
C_maccsf = (1-beta)*G_maccsf/Pmue_csf;
C_picusf = beta*G_picusf/Ppue_usf;
C_piccsf = (1-beta)*G_piccsf/Ppue_csf;

%% Per-user capacities; eqs.(32)-(35)
SE_macUsf = lambda*C_macusf/Pmue_usf;
SE_macCsf = lambda*C_maccsf/Pmue_csf;
SE_picUsf = lambdaPrime*C_picusf/Ppue_usf;
SE_picCsf = lambdaPrime*C_piccsf/Ppue_csf;

%% equate any NaNs to zero
% C_macusf(isnan(C_macusf)) = 0;
% C_maccsf(isnan(C_maccsf)) = 0;
% C_picusf(isnan(C_picusf)) = 0;
% C_piccsf(isnan(C_piccsf)) = 0;
SE_macUsf(isnan(SE_macUsf)) = 0;
SE_macCsf(isnan(SE_macCsf)) = 0;
SE_picUsf(isnan(SE_picUsf)) = 0;
SE_picCsf(isnan(SE_picCsf)) = 0;

%% Output
PerUserSE.SE_macUsf = SE_macUsf;
PerUserSE.SE_macCsf = SE_macCsf;
PerUserSE.SE_picUsf = SE_picUsf;
PerUserSE.SE_picCsf = SE_picCsf;

end