% *************************************************************************
%
% Copyright: (c)  Florida International University. All rights reserved.
%
% MODULE:         SpectralEfficiency_Sim.m
%
% COMPILER:       Matlab R2010a.
%
% LANGUAGE:       Matlab Interpreter.
%
% AUTHOR:         Arvind Merwaday, Florida International Unversity
%
% ABSTRACT:       This function Evaluates the per-user SEs of USF/CSF-MUEs
%                 and USF/CSF-PUEs through Monte Carlo simulation according
%                 to the procedure described in Section 5.1 of the paper.
%
% TIME-STAMP:     21 Mar. 2014
%
% =========================================================================
% Input Parameters:
% =========================================================================
% None
% 
% =========================================================================
% Output Parameters:
% =========================================================================
% None

function PerUserSE = SpectralEfficiency_Sim(Params)

%% Unpack the input parameters
tau         = Params.tau;
alpha       = Params.alpha;
beta        = Params.beta;
rho         = Params.rho;
rhoPrime    = Params.rhoPrime;
lambda      = Params.lambda;
lambdaPrime = Params.lambdaPrime;
lambda_u    = Params.lambda_u;
Dmin_macro  = Params.Dmin_macro;
Dmin_pico   = Params.Dmin_pico;
P_mac_dBm   = Params.P_mac_dBm;
P_pic_dBm   = Params.P_pic_dBm;
K_m_dB      = Params.K_m_dB;
K_p_dB      = Params.K_p_dB;

%% Additional simulation parameters
Nrun        = 20;     % Number of iterations
A           = 100;    % Simulation area A (in km^2)

%% Initial calculations
Au = 0.5*A;                % Smaller area A_u for the UEs
Nm = round(lambda*A);      % Avg. no. of MBSs in area A
Np = round(lambdaPrime*A); % Avg. no. of PBSs in area A
Nue = round(lambda_u*Au);  % Avg. no. of UEs in area A_u
Pm = 10^(P_mac_dBm/10)/1000 * 10^(K_m_dB/10); % MBS effective Tx power
Pp = 10^(P_pic_dBm/10)/1000 * 10^(K_p_dB/10); % PBS effective Tx power

%% Variable declarations
N_usfmue = [];    % No. of USF-MUEs in every cell
N_csfmue = [];    % No. of CSF-MUEs in every cell
N_usfpue = [];    % No. of USF-PUEs in every cell
N_csfpue = [];    % No. of CSF-PUEs in every cell
Cagg_usfmue = []; % Aggregate capacity of USF-MUEs in every cell
Cagg_csfmue = []; % Aggregate capacity of CSF-MUEs in every cell
Cagg_usfpue = []; % Aggregate capacity of USF-PUEs in every cell
Cagg_csfpue = []; % Aggregate capacity of CSF-PUEs in every cell

for nn = 1:Nrun
    %% Generate X- and Y-coordinates of MBSs, PBSs and UEs using
    % uniformly distributed random variables
    Xm = sqrt(A)*(rand(Nm,1)-0.5);      % MBS X-coor.
    Ym = sqrt(A)*(rand(Nm,1)-0.5);      % MBS Y-coor.
    Xp = sqrt(A)*(rand(Np,1)-0.5);      % PBS X-coor.
    Yp = sqrt(A)*(rand(Np,1)-0.5);      % PBS Y-coor.
    Xue = sqrt(Au)*(rand(Nue,1)-0.5);   % UE X-coor.
    Yue = sqrt(Au)*(rand(Nue,1)-0.5);   % UE Y-coor.

    % Coordinates of all the macrocell and picocell nodes (the third column
    % indicates whether it is a PBS (=1) or MBS (=2) )
    All_Node_Coor = [Xp Yp ones(size(Xp));...
                     Xm Ym 2*ones(size(Xm))];

    %% Find the distances between every UE (rows) from every BS (cols)
    [Xpm_mat Xue_mat] = meshgrid(All_Node_Coor(:,1),Xue);
    [Ypm_mat Yue_mat] = meshgrid(All_Node_Coor(:,2),Yue);
    D_ue_bs = sqrt((Xpm_mat-Xue_mat).^2 + (Ypm_mat-Yue_mat).^2);
    
    %% Find MOI and POI for every UE
    [D_moi,idx_moi] = min(D_ue_bs(:,(length(Xp)+1):end),[],2);
    idx_moi = idx_moi + length(Xp);
    [D_poi,idx_poi] = min(D_ue_bs(:,1:length(Xp)),[],2);

    %% Exclude the UEs which do not satisfy the min. distance contraints
    idx = (D_moi < Dmin_macro) | (D_poi < Dmin_pico);
    Xue(idx) = [];
    D_ue_bs(idx,:) = [];
    idx_moi(idx) = [];
    idx_poi(idx) = [];

%     % Plot the MBS, PBS and UE layout
%     Yue(idx) = [];
%     plot(Xm,Ym,'or',Xp,Yp,'.',Xue,Yue,'xg');
%     xlabel('X-axis distance (km)');
%     ylabel('Y-axis distance (km)');
%     legend('Macro nodes','Pico nodes','UEs');

    %% Calculte Received powers at all the UEs from every BS
    Ppm_mat = [Pp*ones(length(Xue),length(Xp)) Pm*ones(length(Xue),length(Xm))]; % Matrix of MBS and PBS transmit powers
    Pr_usf = Ppm_mat.*exprnd(1,size(Ppm_mat))./(D_ue_bs*1e3).^4; % RSRPs during USFs with path-loss exponent = 4
    mult = binornd(1,beta,1,length(Xm)); % each interfering MBS transmits USFs with probability beta
    mult(mult == 0) = alpha; % and CSFs with probability (1-beta). See section 3.1 and Table 1 in the paper
    Pr = Pr_usf.*[ones(length(Xue),length(Xp)) repmat(mult,length(Xue),1)]; % Interference powers from each BS

    %% Find the received powers from MOI and POI for each UE
    P_mac = Pr(sub2ind(size(Pr),(1:length(idx_moi))',idx_moi));
    P_pic = Pr(sub2ind(size(Pr),(1:length(idx_poi))',idx_poi));
    P_mac_usf = Pr_usf(sub2ind(size(Pr_usf),(1:length(idx_moi))',idx_moi));

    %% Calculate the total interference at each UE
    Z = sum(Pr,2) - P_mac - P_pic;

    %% Calculte the macro and pico SIRs according to eqs. (2)-(5) in the paper
    SIR_mac_usf = P_mac_usf./(P_pic + Z);
    SIR_pic_usf = P_pic./(P_mac_usf + Z);
    SIR_mac_csf = alpha*SIR_mac_usf;
    SIR_pic_csf = P_pic./(alpha*P_mac_usf + Z);

    %% Cell selection and scheduling conditions as per eqs. (6)-(9) in the paper
    idx_mue_usf = (SIR_mac_usf > tau*SIR_pic_usf) & (SIR_mac_usf <= rho);
    idx_mue_csf = (SIR_mac_usf > tau*SIR_pic_usf) & (SIR_mac_usf > rho);
    idx_pue_usf = (SIR_mac_usf <= tau*SIR_pic_usf) & (SIR_pic_usf > rhoPrime);
    idx_pue_csf = (SIR_mac_usf <= tau*SIR_pic_usf) & (SIR_pic_usf <= rhoPrime);

    %% Calculate aggregate capacities of USF MUEs in every macrocell
    idx_moi_usf = idx_moi(idx_mue_usf);
    SIR_mac_usf = SIR_mac_usf(idx_mue_usf);
    hm_usf = tabulate([-1; idx_moi_usf]);
    hm_usf(1,:) = [];
    Nmue_usf = hm_usf(:,2);
    Ctot = zeros(size(Nmue_usf));
    for xx = 1:length(Nmue_usf)
        idx = idx_moi_usf == hm_usf(xx,1);
        Ctot(xx) = beta*sum(log2(1+SIR_mac_usf(idx)))/Nmue_usf(xx);
    end
    N_usfmue = [N_usfmue; Nmue_usf];
    Cagg_usfmue = [Cagg_usfmue; Ctot];
    
    %% Calculate aggregate capacities of CSF MUEs in every macrocell
    idx_moi_csf = idx_moi(idx_mue_csf);
    SIR_mac_csf = SIR_mac_csf(idx_mue_csf);
    hm_csf = tabulate([-1; idx_moi_csf]);
    hm_csf(1,:) = [];
    Nmue_csf = hm_csf(:,2);
    Ctot = zeros(size(Nmue_csf));
    for xx = 1:length(Nmue_csf)
        idx = idx_moi_csf == hm_csf(xx,1);
        Ctot(xx) = (1-beta)*sum(log2(1+SIR_mac_csf(idx)))/Nmue_csf(xx);
    end
    N_csfmue = [N_csfmue; Nmue_csf];
    Cagg_csfmue = [Cagg_csfmue; Ctot];
    
    %% Calculate aggregate capacities of USF PUEs in every macrocell
    idx_poi_usf = idx_poi(idx_pue_usf);
    SIR_pic_usf = SIR_pic_usf(idx_pue_usf);
    hp_usf = tabulate([-1; idx_poi_usf]);
    hp_usf(1,:) = [];
    Npue_usf = hp_usf(:,2);
    Ctot = zeros(size(Npue_usf));
    for xx = 1:length(Npue_usf)
        idx = idx_poi_usf == hp_usf(xx,1);
        Ctot(xx) = beta*sum(log2(1+SIR_pic_usf(idx)))/Npue_usf(xx);
    end
    N_usfpue = [N_usfpue; Npue_usf];
    Cagg_usfpue = [Cagg_usfpue; Ctot];
    
    %% Calculate aggregate capacities of CSF PUEs in every macrocell
    idx_poi_csf = idx_poi(idx_pue_csf);
    SIR_pic_csf = SIR_pic_csf(idx_pue_csf);
    hp_csf = tabulate([-1; idx_poi_csf]);
    hp_csf(1,:) = [];
    Npue_csf = hp_csf(:,2);
    Ctot = zeros(size(Npue_csf));
    for xx = 1:length(Npue_csf)
        idx = idx_poi_csf == hp_csf(xx,1);
        Ctot(xx) = (1-beta)*sum(log2(1+SIR_pic_csf(idx)))/Npue_csf(xx);
    end
    N_csfpue = [N_csfpue; Npue_csf];
    Cagg_csfpue = [Cagg_csfpue; Ctot];
end

%% Aggregate capacities
Cagg_usfmue_mean = mean(Cagg_usfmue);
Cagg_csfmue_mean = mean(Cagg_csfmue);
Cagg_usfpue_mean = mean(Cagg_usfpue);
Cagg_csfpue_mean = mean(Cagg_csfpue);

%% Mean number of USF/CSF-MUEs and USF/CSF-PUEs in a cell (normalized with 1/lambda_u)
N_usfmue_mean = mean(N_usfmue)/lambda_u;
N_csfmue_mean = mean(N_csfmue)/lambda_u;
N_usfpue_mean = mean(N_usfpue)/lambda_u;
N_csfpue_mean = mean(N_csfpue)/lambda_u;

%% Per-user SEs ( Normalized with lambda_u)
SE_macUsf = Cagg_usfmue_mean/N_usfmue_mean;
SE_macCsf = Cagg_csfmue_mean/N_csfmue_mean;
SE_picUsf = Cagg_usfpue_mean/N_usfpue_mean;
SE_picCsf = Cagg_csfpue_mean/N_csfpue_mean;

%% Output
PerUserSE.SE_macUsf = SE_macUsf;
PerUserSE.SE_macCsf = SE_macCsf;
PerUserSE.SE_picUsf = SE_picUsf;
PerUserSE.SE_picCsf = SE_picCsf;

end