%% ************************************************************************
%
% Copyright: (c)  Florida International University. All rights reserved.
%
% MODULE:         main.m
%
% COMPILER:       Matlab R2010a.
%
% LANGUAGE:       Matlab Interpreter.
%
% AUTHOR:         Arvind Merwaday, Florida International Unversity
%
% ABSTRACT:       This function calculates the per-user SEs of MUEs and 
%                 PUEs. Plots the SE results for different values of REB 
%                 and power reduction factor.
%
% TIME-STAMP:     23 Mar. 2014
%
% =========================================================================
% Input Parameters:
% =========================================================================
% None
% 
% =========================================================================
% Output Parameters:
% =========================================================================
% None

function main()

%% =============== Parameter settings =====================================
tau_dB          = 0:3:15;       % Range expansion bias (in dB)
alpha           = [0 0.5 1];    % Power reduction factor [0,1]
beta            = 0.5;          % USF duty cycle [0,1]
rho_dB          = 4;            % MUE scheduling threshold (in dB)
rhoPrime_dB     = 0;            % PUE scheduling threshold (in dB)
lambda          = 4.6;          % MBS density (in MBSs/km^2)
lambdaPrime     = lambda*3;     % PBS density (in PBSs/km^2)
lambda_u        = 200;          % UE density (in UEs/km^2)
Dmin_macro      = 0.035;        % Min. distance between MBS and UE (in km)
Dmin_pico       = 0.010;        % Min. distance between PBS and UE (in km)
P_mac_dBm       = 46;           % MBS Transmit power (in dBm)
P_pic_dBm       = 30;           % PBS Transmit power (in dBm)
K_m_dB          = -11;          % Attenuation due to geometrical parameters of MBS antenna (in dB)
K_p_dB          = -11;          % Attenuation due to geometrical parameters of PBS antenna (in dB)
%============== End Parameter settings ====================================

%% Convert dB to absolute values
tau = 10.^(tau_dB/10);
rho = 10.^(rho_dB/10);
rhoPrime = 10.^(rhoPrime_dB/10);

%% Variables for Monte-Carlo simulation results
SE_macUsf_sim = zeros(length(tau),length(alpha));
SE_macCsf_sim = SE_macUsf_sim;
SE_picUsf_sim = SE_macUsf_sim;
SE_picCsf_sim = SE_macUsf_sim;

%% Variables for PPP results
SE_macUsf_ppp = zeros(length(tau),length(alpha));
SE_macCsf_ppp = SE_macUsf_ppp;
SE_picUsf_ppp = SE_macUsf_ppp;
SE_picCsf_ppp = SE_macUsf_ppp;

%% Create a structure for parameter inputs
Params.tau          = tau(1);
Params.alpha        = alpha(1);
Params.beta         = beta;
Params.rho          = rho;
Params.rhoPrime     = rhoPrime;
Params.lambda       = lambda;
Params.lambdaPrime  = lambdaPrime;
Params.lambda_u     = lambda_u;
Params.Dmin_macro   = Dmin_macro;
Params.Dmin_pico    = Dmin_pico;
Params.P_mac_dBm    = P_mac_dBm;
Params.P_pic_dBm    = P_pic_dBm;
Params.K_m_dB       = K_m_dB;
Params.K_p_dB       = K_p_dB;

%% Evaluate Simulation and PPP results
for n = 1:length(alpha)
    for k = 1:length(tau)
        %% Set REB and power reduction factor
        Params.tau = tau(k);
        Params.alpha = alpha(n);
        
        %% Calculate per-user spectral efficiencies (SEs) with Monte Carlo simulation
        PerUserSE = SpectralEfficiency_Sim(Params);
        SE_macUsf_sim(k,n) = PerUserSE.SE_macUsf;
        SE_macCsf_sim(k,n) = PerUserSE.SE_macCsf;
        SE_picUsf_sim(k,n) = PerUserSE.SE_picUsf;
        SE_picCsf_sim(k,n) = PerUserSE.SE_picCsf;
        
        %% Calculate per-user spectral efficiencies (SEs) with Monte Carlo simulation
        PerUserSE = SpectralEfficiency_PPP(Params);
        SE_macUsf_ppp(k,n) = PerUserSE.SE_macUsf;
        SE_macCsf_ppp(k,n) = PerUserSE.SE_macCsf;
        SE_picUsf_ppp(k,n) = PerUserSE.SE_picUsf;
        SE_picCsf_ppp(k,n) = PerUserSE.SE_picCsf;
    end
end

%% Plot per-user SEs of USF-MUEs and CSF-MUEs v/s REB for different alpha values
figure(1);
plot(tau_dB,SE_macUsf_sim(:,1),'o','LineWidth',2);
hold on;
plot(tau_dB,SE_macUsf_ppp(:,1),'LineWidth',2);
plot(tau_dB,SE_macUsf_sim(:,2),'sr','LineWidth',2);
plot(tau_dB,SE_macUsf_ppp(:,2),'--r','LineWidth',2);
plot(tau_dB,SE_macUsf_sim(:,3),'dg','LineWidth',2);
plot(tau_dB,SE_macUsf_ppp(:,3),':g','LineWidth',2);
plot(tau_dB,SE_macCsf_sim(:,2),'sr','LineWidth',2);
plot(tau_dB,SE_macCsf_ppp(:,2),'--r','LineWidth',2);
plot(tau_dB,SE_macCsf_sim(:,3),'dg','LineWidth',2);
plot(tau_dB,SE_macCsf_ppp(:,3),':g','LineWidth',2);
hold off;
xlabel('Range expansion bias, \tau (in dB)');
ylabel('Per-user macrocell SE x \lambda_u (bps/hertz)');
legend('\alpha = 0 (Sim)','\alpha = 0 (PPP)','\alpha = 0.5 (Sim)','\alpha = 0.5 (PPP)','\alpha = 1 (Sim)','\alpha = 1 (PPP)');
title(['\rho = ' num2str(rho_dB) ' dB,  ' '\beta = ' num2str(beta) ',  \lambda = ' num2str(lambda) ' MBSs/km^2,  \lambda\prime = ' num2str(lambdaPrime) ' PBSs/km^2']);

%% Plot per-user SEs of USF-PUEs and CSF-PUEs v/s REB for different alpha values
figure(2);
plot(tau_dB,SE_picUsf_sim(:,1),'o','LineWidth',2);
hold on;
plot(tau_dB,SE_picUsf_ppp(:,1),'LineWidth',2);
plot(tau_dB,SE_picUsf_sim(:,2),'sr','LineWidth',2);
plot(tau_dB,SE_picUsf_ppp(:,2),'--r','LineWidth',2);
plot(tau_dB,SE_picUsf_sim(:,3),'dg','LineWidth',2);
plot(tau_dB,SE_picUsf_ppp(:,3),':g','LineWidth',2);
plot(tau_dB,SE_picCsf_sim(:,1),'o','LineWidth',2);
plot(tau_dB,SE_picCsf_ppp(:,1),'LineWidth',2);
plot(tau_dB,SE_picCsf_sim(:,2),'sr','LineWidth',2);
plot(tau_dB,SE_picCsf_ppp(:,2),'--r','LineWidth',2);
plot(tau_dB,SE_picCsf_sim(:,3),'dg','LineWidth',2);
plot(tau_dB,SE_picCsf_ppp(:,3),':g','LineWidth',2);
hold off;
xlabel('Range expansion bias, \tau (in dB)');
ylabel('Per-user picocell SE x \lambda_u (bps/hertz)');
legend('\alpha = 0 (Sim)','\alpha = 0 (PPP)','\alpha = 0.5 (Sim)','\alpha = 0.5 (PPP)','\alpha = 1 (Sim)','\alpha = 1 (PPP)');
title(['\rho\prime = ' num2str(rhoPrime_dB) ' dB,  ' '\beta = ' num2str(beta) ',  \lambda = ' num2str(lambda) ' MBSs/km^2,  \lambda\prime = ' num2str(lambdaPrime) ' PBSs/km^2']);

end